
package dust

import (
	"context"
	"fmt"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
	"smart-site-management/errors"
	"smart-site-management/model"
	"smart-site-management/pb/v1"
)

type DustInfo struct {
	Pm10Avg              float64  `protobuf:"fixed64,1,opt,name=pm10_avg,json=pm10Avg,proto3" json:"pm10"`
	Pm25Avg              float64  `protobuf:"fixed64,2,opt,name=pm25_avg,json=pm25Avg,proto3" json:"pm25"`
	St                   string   `protobuf:"bytes,3,opt,name=st,proto3" json:"st"`
	Cn                   string   `protobuf:"bytes,4,opt,name=cn,proto3" json:"cn"`
	Mn                   string   `protobuf:"bytes,5,opt,name=mn,proto3" json:"mn"`
	Time                 string   `protobuf:"bytes,6,opt,name=time,proto3" json:"time"`
	TspAvg               float64  `protobuf:"fixed64,7,opt,name=tsp_avg,json=tspAvg,proto3" json:"tsp"`
	T01Avg               float64  `protobuf:"fixed64,8,opt,name=t01_avg,json=t01Avg,proto3" json:"t01"`
	H01Avg               float64  `protobuf:"fixed64,9,opt,name=h01_avg,json=h01Avg,proto3" json:"h01"`
	W01Avg               float64  `protobuf:"fixed64,10,opt,name=w01_avg,json=w01Avg,proto3" json:"w01"`
	W02Avg               float64  `protobuf:"fixed64,11,opt,name=w02_avg,json=w02Avg,proto3" json:"w02"`
	R01Avg               float64  `protobuf:"fixed64,12,opt,name=r01_avg,json=r01Avg,proto3" json:"r01"`
	P01Avg               float64  `protobuf:"fixed64,13,opt,name=p01_avg,json=p01Avg,proto3" json:"p01"`
	B03Avg               float64  `protobuf:"fixed64,14,opt,name=b03_avg,json=b03Avg,proto3" json:"b03"`
	Lng                  float64  `protobuf:"fixed64,15,opt,name=lng,proto3" json:"lng"`
	Lat                  float64  `protobuf:"fixed64,16,opt,name=lat,proto3" json:"lat"`
	Pm10Flag             string   `protobuf:"bytes,17,opt,name=pm10_flag,json=pm10Flag,proto3" json:"pm10_flag"`
	Pm25Flag             string   `protobuf:"bytes,18,opt,name=pm25_flag,json=pm25Flag,proto3" json:"pm25_flag"`
	TspFlag              string   `protobuf:"bytes,19,opt,name=tsp_flag,json=tspFlag,proto3" json:"tsp_flag"`
	T01Flag              string   `protobuf:"bytes,20,opt,name=t01_flag,json=t01Flag,proto3" json:"t01_flag"`
	H01Flag              string   `protobuf:"bytes,21,opt,name=h01_flag,json=h01Flag,proto3" json:"h01_flag"`
	W01Flag              string   `protobuf:"bytes,22,opt,name=w01_flag,json=w01Flag,proto3" json:"w01_flag"`
	W02Flag              string   `protobuf:"bytes,23,opt,name=w02_flag,json=w02Flag,proto3" json:"w02_flag"`
	R01Flag              string   `protobuf:"bytes,24,opt,name=r01_flag,json=r01Flag,proto3" json:"r01_flag"`
	P01Flag              string   `protobuf:"bytes,25,opt,name=p01_flag,json=p01Flag,proto3" json:"p01_flag"`
	B03Flag              string   `protobuf:"bytes,26,opt,name=b03_flag,json=b03Flag,proto3" json:"b03_flag"`
	W03 float64 `json:"w03"`
}

type DustInfoMean struct {
	Pm10Avg              float64  `protobuf:"fixed64,1,opt,name=pm10_avg,json=pm10Avg,proto3" json:"mean_pm10"`
	Pm25Avg              float64  `protobuf:"fixed64,2,opt,name=pm25_avg,json=pm25Avg,proto3" json:"mean_pm25"`
	Time                 string   `protobuf:"bytes,6,opt,name=time,proto3" json:"time"`
	TspAvg               float64  `protobuf:"fixed64,7,opt,name=tsp_avg,json=tspAvg,proto3" json:"mean_tsp"`
	T01Avg               float64  `protobuf:"fixed64,8,opt,name=t01_avg,json=t01Avg,proto3" json:"mean_t01"`
	H01Avg               float64  `protobuf:"fixed64,9,opt,name=h01_avg,json=h01Avg,proto3" json:"mean_h01"`
	W01Avg               float64  `protobuf:"fixed64,10,opt,name=w01_avg,json=w01Avg,proto3" json:"mean_w01"`
	W02Avg               float64  `protobuf:"fixed64,11,opt,name=w02_avg,json=w02Avg,proto3" json:"mean_w02"`
	R01Avg               float64  `protobuf:"fixed64,12,opt,name=r01_avg,json=r01Avg,proto3" json:"mean_r01"`
	P01Avg               float64  `protobuf:"fixed64,13,opt,name=p01_avg,json=p01Avg,proto3" json:"mean_p01"`
	B03Avg               float64  `protobuf:"fixed64,14,opt,name=b03_avg,json=b03Avg,proto3" json:"mean_b03"`
	Lng                  float64  `protobuf:"fixed64,15,opt,name=lng,proto3" json:"mean_lng"`
	Lat                  float64  `protobuf:"fixed64,16,opt,name=lat,proto3" json:"mean_lat"`
	W03 float64 `json:"mean_w03"`
}

func getDustDbName(dtype int32) string {
	tail := "2011"
	prefix := "dust_monitor_"
	switch dtype {
	case 0:
		tail = "2011"
	case 1:
		tail = "2061"
	case 2:
		tail = "2031"
	}
	return prefix + tail
}

func LatestDustMonitorData(ctx context.Context, req *v1.LatestDustMonitorDataRequest)(reply *v1.LatestDustMonitorDataReply, err error) {

	if req.Sn == "" {
		// 获取一个扬尘设备
		p := model.TDevice{}
		where := map[string]interface{}{
			"project_id":req.ProjectId,
			"device_code":model.DeviceTypeDust,
			"verify_status":1,
			"status":1,
		}
		err = p.Find(database.DB(), where)
		if err != nil {
			return nil, errors.DataBaseError
		}
		if p.Id == 0 {
			return &v1.LatestDustMonitorDataReply{Data:&v1.DustMonitorData{}}, nil
		}
		req.Sn = p.Sn
	}


	dbname := getDustDbName(req.Type)
	array := []DustInfo{}
	sql := fmt.Sprintf("select * from \"%s\" order by time desc limit 1 tz('Asia/Shanghai')", req.Sn)
	_, err = model.QueryInfluxdb(sql, dbname, &array)
	if err != nil {
		logger.Error("LatestDustMonitorData",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	reply = &v1.LatestDustMonitorDataReply{}
	if len(array) == 0 {
		return &v1.LatestDustMonitorDataReply{Data:&v1.DustMonitorData{}}, nil
	}
	reply.Data = &v1.DustMonitorData{}
	reply.Data.Date = array[0].Time
	reply.Data.B03 = array[0].B03Avg
	reply.Data.H01 = array[0].H01Avg
	reply.Data.P01 = array[0].P01Avg
	reply.Data.Pm10 = array[0].Pm10Avg
	reply.Data.Pm25 = array[0].Pm25Avg
	reply.Data.R01 = array[0].R01Avg
	reply.Data.T01 = array[0].T01Avg
	reply.Data.Tsp = array[0].TspAvg
	reply.Data.W01 = array[0].W01Avg
	reply.Data.W02 = array[0].W02Avg
	return reply, nil
}
