// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package model

import (
	"encoding/json"
	"fmt"
	"smart-site-management/consts"
	"smart-site-management/errors"
	"time"

	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"

	"github.com/jinzhu/gorm"
)

type TAlarmRule struct {
	Id             int64  `gorm:"primary_key"`
	ProjectId      int64  `gorm:"column:project_id"`
	Sn             string `gorm:"sn"`
	ContinuePeriod int32  `gorm:"column:continue_period"`
	AlarmCount     int32  `gorm:"column:alarm_count"`
	SilencePeriod  int32  `gorm:"column:silence_period"`
	IsOn           bool   `gorm:"column:is_on"`
	CreatedAt      string `gorm:"column:created_at"`
	UpdatedAt      string `gorm:"column:updated_at"`
}

func (TAlarmRule) TableName() string {
	return "t_alarm_rule"
}

func (p *TAlarmRule) Insert(db *gorm.DB) error {
	timeNow := time.Now().Format(consts.TimeSecondLayOut)
	p.CreatedAt = timeNow
	p.UpdatedAt = timeNow
	err := db.Create(p).Error
	if err != nil {
		fields, _ := json.Marshal(*p)
		logger.Error("mysql",
			zap.String("sql", "insert into t_alarm_rule"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TAlarmRule) Delete(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Delete(p).Error
	if err != nil {
		fields, _ := json.Marshal(filter)
		logger.Error("mysql",
			zap.String("sql", "delete from t_alarm_rule"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
	}
	return err
}

// 通过结构体变量更新字段值, gorm库会忽略零值字段。就是字段值等于0, nil, "", false这些值会被忽略掉，不会更新。如果想更新零值，可以使用map类型替代结构体。
func (p *TAlarmRule) UpdateSome(db *gorm.DB, filed map[string]interface{}) error {
	if filed == nil {
		return errors.ParamsError
	}
	timeNow := time.Now().Format(consts.TimeSecondLayOut)
	filed["updated_at"] = timeNow
	err := db.Model(p).Updates(filed).Error
	if err != nil {
		fields, _ := json.Marshal(filed)
		logger.Error("mysql",
			zap.String("sql", "update t_alarm_rule"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TAlarmRule) Query(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Find(p).Error
	if err != nil {
		fields, _ := json.Marshal(filter)
		logger.Error("mysql",
			zap.String("sql", "select from t_alarm_rule"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TAlarmRule) QueryAll(db *gorm.DB, filter map[string]interface{}) (list []TAlarmRule, err error) {
	err = db.Where(filter).Find(&list).Error
	if err != nil {
		fields, _ := json.Marshal(filter)
		logger.Error("mysql",
			zap.String("sql", "select from t_alarm_rule"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
	}
	return list, err
}

type AlarmRuleListItem struct {
	TAlarmRule
	DeviceName string
	ProjectName string
}

func (p *TAlarmRule) List(db *gorm.DB, sn string, projectId int64, page int32) (list []AlarmRuleListItem, total int64, err error) {
	sql := fmt.Sprintf("select t1.*, t2.name as project_name, t3.name as device_name from t_alarm_rule as t1 left join t_project as t2 on t1.project_id=t2.id left join t_device as t3 on t3.sn=t1.sn and t3.project_id=t1.project_id")
	countSql := fmt.Sprintf("select count(1) as count from t_alarm_rule as t1 left join t_project as t2 on t1.project_id=t2.id left join t_device as t3 on t3.sn=t1.sn and t3.project_id=t1.project_id")

	whereArray := []string{}
	if sn != "" {
		whereArray = append(whereArray, fmt.Sprintf("t1.sn='%s'", sn))
	}
	if projectId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.project_id=%d", projectId))
	}
	where := ""
	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf("where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}
	sql = fmt.Sprintf("%s %s limit %d offset %d", sql, where, PageSize, int(page - 1) *PageSize)
	countSql = fmt.Sprintf("%s %s", countSql, where)


	type ResultCount struct {
		Count int64
	}
	array := []ResultCount{}
	ret := []AlarmRuleListItem{}

	err = db.Raw(countSql).Scan(&array).Error

	if err != nil {
		logger.Error("mysql",
			zap.String("sql", countSql),
			zap.String("error", err.Error()))
		return  nil, 0, errors.DataBaseError
	}
	if len(array) == 0 {
		return nil, 0, nil
	}
	if array[0].Count == 0 {
		return nil, 0, nil
	}

	err = db.Raw(sql).Scan(&ret).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("error", err.Error()))
		return nil, array[0].Count, errors.DataBaseError
	}
	return ret, array[0].Count, nil
}

func (p *TAlarmRule) Count(db *gorm.DB, where map[string]interface{}, or map[string]interface{}) (int64, error) {
	if len(where) > 0 || len(or) > 0{
		cond, val, err := whereBuildAndOr(where, or)
		if err != nil {
			return 0, err
		}
		ret := int64(0)
		err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error
		if err != nil {
			fields, _ := json.Marshal(val)
			logger.Error("mysql",
				zap.String("sql", "select count(1) t_alarm_rule"),
				zap.String("fields", cond + ","+string(fields)),
				zap.String("error", err.Error()))
			return  ret, errors.DataBaseError
		}

		return ret, err
	}
	ret := int64(0)
	err := db.Table(p.TableName()).Count(&ret).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select count(1) t_project"),
			zap.String("error", err.Error()))
		return  ret, errors.DataBaseError
	}
	return ret, err
}
