// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package model

import (
	"go.uber.org/zap"
	"smart-site-management/errors"
	"time"
	"fmt"
	"github.com/jinzhu/gorm"
	"github.com/jaryhe/gopkgs/logger"
	"encoding/json"
)

const (
	DeviceTypeVedio = 9
	DeviceTypeDust = 5
	DeviceTypeTower = 1
	DeviceTypeHighFramework = 7
	DeviceTypeUnloading = 6
	DeviceTypeUpDown = 4
	DeviceTypeWireless = 8

)
type TDevice struct {
	Id int64 `gorm:"column:id" json:"id" form:"id"`
	ProjectId int64 `gorm:"column:project_id" json:"project_id" form:"project_id"`
	ProviderId int64 `gorm:"column:provider_id" json:"provider_id" form:"provider_id"`
	Name string `gorm:"column:name" json:"name" form:"name"`
	Sn string `gorm:"column:sn" json:"sn" form:"sn"`
	Addr string `gorm:"column:addr" json:"addr" form:"addr"`
	DeviceCode int32 `gorm:"column:device_code" json:"device_code" form:"device_code"`
	Lon float64 `gorm:"column:lon" json:"lon" form:"lon"`
	Lat float64 `gorm:"column:lat" json:"lat" form:"lat"`
	XCoord float64 `gorm:"column:x_coord" json:"x_coord" form:"x_coord"`
	YCoord float64 `gorm:"column:y_coord" json:"y_coord" form:"y_coord"`
	Status int64 `gorm:"column:status" json:"status" form:"status"`
	CreatedAt time.Time `gorm:"column:created_at" json:"created_at" form:"created_at"`
	UpdatedAt time.Time `gorm:"column:updated_at" json:"updated_at" form:"updated_at"`
	VerifyStatus int64 `gorm:"column:verify_status" json:"verify_status" form:"verify_status"`
	Key string `gorm:"column:key" json:"key" form:"key"`
	VssChannelId int64 `json:"vss_channel_id"`
}

func (TDevice) TableName() string {
	return "t_device"
}

func (p *TDevice) Insert(db *gorm.DB) error {
	err := db.Create(p).Error
	if err != nil {
		fields, _ := json.Marshal(p)
		logger.Error("mysql",
			zap.String("sql", "insert info t_device "),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}

func (p *TDevice) List(db *gorm.DB, where map[string]interface{}, page int32, pageSize int32) (list []TDevice, err error) {
	offset := (page - 1) *pageSize
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return list, err
		}

		err = db.Table(p.TableName()).Where(cond, val...).Limit(pageSize).Offset(offset).Find(&list).Error
		if err != nil {
			fields, _ := json.Marshal(where)
			logger.Error("mysql",
				zap.String("sql", "select * from t_device"),
				zap.String("fields", string(fields)),
				zap.String("error", err.Error()))
			if err == gorm.ErrRecordNotFound {
				return list, nil
			}
			return  list, errors.DataBaseError
		}

		return list, err
	}
	err = db.Table(p.TableName()).Limit(pageSize).Offset(offset).Find(&list).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select * from t_device"),
			zap.String("error", err.Error()))
		if err == gorm.ErrRecordNotFound {
			return list, nil
		}
		return  list, errors.DataBaseError
	}

	return list, err

}

// 按设备类型统计
func (p *TDevice) Statistic(db *gorm.DB, projectId int64, deviceType int32, result interface{}) error {

	sql := fmt.Sprintf("select device_code as device_type, sum(status=0) as offline, sum(status=1) as online from t_device where project_id=%d group by type", projectId)
	if deviceType > 0 {
		sql = fmt.Sprintf("select device_code as device_type, sum(status=0) as offline, sum(status=1) as online from t_device where project_id=%d and type = %d group by type", projectId, deviceType)
	}
	err := db.Raw(sql).Scan(result).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}

func (p *TDevice) Del(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	err = db.Table(p.TableName()).Where(cond, val...).Delete(p).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "delete from t_device "),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}

func (p *TDevice) Find(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	err =  db.Table(p.TableName()).Where(cond, val...).First(p).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "select from t_device "),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		if err == gorm.ErrRecordNotFound {
			return nil
		}
		return  errors.DataBaseError
	}
	return err
}

func (p *TDevice) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}

	err = db.Table(p.TableName()).Where(cond, val...).Updates(values).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		updateValues, _ := json.Marshal(values)
		logger.Error("mysql",
			zap.String("sql", "update t_device "),
			zap.String("fields", string(fields)),
			zap.String("values", string(updateValues)),
			zap.String("error", err.Error()))
		return  errors.DataBaseError
	}
	return err
}

func (p *TDevice) Count(db *gorm.DB, where map[string]interface{}) (int64, error) {
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return 0, err
		}
		ret := int64(0)
		err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error
		if err != nil {
			fields, _ := json.Marshal(where)
			logger.Error("mysql",
				zap.String("sql", "select count(1) from t_device "),
				zap.String("fields", string(fields)),
				zap.String("error", err.Error()))
			return  ret, errors.DataBaseError
		}
		return ret, err
	}
	ret := int64(0)
	err := db.Table(p.TableName()).Count(&ret).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "select count(1) from t_device "),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return  ret, errors.DataBaseError
	}
	return ret, err
}

type DeviceItem struct {
	Id                   int64
	Sn                   string
	TypeCode             int32
	TypeName             string
	ProjectName          string
	ProjectId            int64
	SafetyRecordNo       string
	CreatedTime          time.Time
	Status               int32
	State int32
	ProjectApproveTime time.Time
	Key string
	ProviderName string
	SocialCode string
	Name string
	Ip string
	Port int
	MediaTransport string
	ChannelCount int
	ChannelName string
	ChannelNo string

}

func listRawSqlForVedio(req ListRawRequest)(string, string, []interface{}) {
	sql := fmt.Sprintf("select t1.id as id, t1.key, t1.name as name, t1.sn as sn, t1.device_code as type_code, t1.project_id as project_id, t1.project_approve_time, t4.MediaTransport as media_transport, t4.IP as ip, t4.Port as port, t4.channel_count, "+
		"t1.created_at as created_time, t4.Alive as state, t1.verify_status as status, t2.safety_record_no, t2.name as project_name, t3.name as provider_name, t3.social_code from t_device as t1 left join t_project as t2 on t1.project_id=t2.id left join t_provider as t3 on t1.provider_id=t3.id left join VSSDevTbl as t4 on t4.DevPubID=t1.sn")
	countSql := fmt.Sprintf("select count(1) as count from t_device as t1 left join t_project as t2 on t1.project_id=t2.id ")
	args := []interface{}{}
	whereArray := []string{}
	if req.ProviderId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.provider_id=%d", req.ProviderId))
	}

	whereArray = append(whereArray, fmt.Sprintf("t1.device_code = %d", DeviceTypeVedio))

	if req.Filter != "" {
		whereArray = append(whereArray, fmt.Sprintf("(t1.sn like '%%%s%%' or t2.safety_record_no like '%%%s%%' or t2.name like '%%%s%%')", req.Filter, req.Filter, req.Filter))
	}

	if req.ProjectId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.project_id = %d", req.ProjectId))
	}
	if req.CanDel {
		req.IsAll = true
		req.StatusFilter = []int32{1}
		whereArray = append(whereArray, fmt.Sprintf("(select count(1) from t_project_job where device_id=t1.id and type = 1 and (status = 0 or status = 1)) = 0"))
	}
	if len(req.StatusFilter) > 0 {
		args = append(args, req.StatusFilter)
		whereArray = append(whereArray, fmt.Sprintf("t1.verify_status in(?)"))
	}
	where := ""

	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf(" where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}
	offset := int64(req.Page - 1) *(req.PageSize)
	if req.IsAll {
		sql = fmt.Sprintf("%s %s", sql, where)
		countSql = fmt.Sprintf("%s %s", countSql, where)
	} else {
		sql = fmt.Sprintf("%s %s limit %d offset %d", sql, where, req.PageSize, offset)
		countSql = fmt.Sprintf("%s %s", countSql, where)
	}
	return sql, countSql, args
}


func listRawSql(req ListRawRequest)(string, string, []interface{}) {
	if req.IsVedio {
		return listRawSqlForVedio(req)
	}
	sql := fmt.Sprintf("select t1.id as id, t1.name, t4.NickName as channel_name, t4.ChanPubID as channel_no, t1.key, t1.sn as sn, t1.device_code as type_code, t1.project_id as project_id, t1.project_approve_time, "+
		"t1.created_at as created_time, t1.status as state, t1.verify_status as status, t2.safety_record_no, t2.name as project_name, t3.name as provider_name, t3.social_code from t_device as t1 left join t_project as t2 on t1.project_id=t2.id left join t_provider as t3 on t1.provider_id=t3.id left join VSSChannelTbl as t4 on t1.vss_channel_id = t4.ID")
	countSql := fmt.Sprintf("select count(1) as count from t_device as t1 left join t_project as t2 on t1.project_id=t2.id ")
	args := []interface{}{}
	whereArray := []string{}
	if req.ProviderId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.provider_id=%d", req.ProviderId))
	}
	if req.TypeCode > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.device_code = %d", req.TypeCode))
	}
	if !req.CanDel {
		whereArray = append(whereArray, fmt.Sprintf("t1.device_code <> %d", DeviceTypeVedio))
	}
	if req.Filter != "" {
		whereArray = append(whereArray, fmt.Sprintf("(t1.sn like '%%%s%%' or t2.safety_record_no like '%%%s%%' or t2.name like '%%%s%%')", req.Filter, req.Filter, req.Filter))
	}

	if req.ProjectId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.project_id = %d", req.ProjectId))
	}
	if req.CanDel {
		req.IsAll = true
		req.StatusFilter = []int32{1}
		whereArray = append(whereArray, fmt.Sprintf("(select count(1) from t_project_job where device_id=t1.id and type = 1 and (status = 0 or status = 1)) = 0"))
	}
	if len(req.StatusFilter) > 0 {
		args = append(args, req.StatusFilter)
		whereArray = append(whereArray, fmt.Sprintf("t1.verify_status in(?)"))
	}
	where := ""

	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf(" where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}
	offset := int64(req.Page - 1) *(req.PageSize)
	if req.IsAll {
		sql = fmt.Sprintf("%s %s order by t1.created_at desc", sql, where)
		countSql = fmt.Sprintf("%s %s", countSql, where)
	} else {
		sql = fmt.Sprintf("%s %s order by t1.created_at desc limit %d offset %d", sql, where, req.PageSize, offset)
		countSql = fmt.Sprintf("%s %s", countSql, where)
	}
	return sql, countSql, args
}

type ListRawRequest struct {
	ProviderId int64
	Filter string
	StatusFilter []int32
	IsAll bool
	CanDel bool
	Page int32
	TypeCode int32
	ProjectId int64
	IsVedio bool
	PageSize int64
}

func (p *TDevice) ListRaw(db *gorm.DB, req ListRawRequest) ([]DeviceItem, int64, error) {
	type ResultCount struct {
		Count int64
	}
	if req.PageSize < 1 {
		req.PageSize = 10
	}
	array := []ResultCount{}
	ret := []DeviceItem{}
	var err error

	sql, countSql, args := listRawSql(req)

	err = db.Raw(countSql, args...).Scan(&array).Error

	if err != nil {
		fields, _ := json.Marshal(args)
		logger.Error("mysql",
			zap.String("sql", countSql),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return  nil, 0, errors.DataBaseError
	}
	if len(array) == 0 {
		return nil, 0, nil
	}
	if array[0].Count == 0 {
		return nil, 0, nil
	}

	err = db.Raw(sql, args...).Scan(&ret).Error
	if err != nil {
		fields, _ := json.Marshal(args)
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return nil, array[0].Count, errors.DataBaseError
	}
	return ret, array[0].Count, nil
}

type DeviceStatisticsItem struct {
	TypeCode int32
	Total int32
	Online int32
}

func (p *TDevice) StatisticWithProject(db *gorm.DB, projectId int64) ([]DeviceStatisticsItem, error) {
	sql := fmt.Sprintf("select device_code as type_code, sum(1) as total, sum(status=1) as online from t_device where project_id=? and verify_status=1 and device_code <> %d group by device_code", DeviceTypeVedio)
	array := []DeviceStatisticsItem{}
	err := db.Raw(sql, projectId).Scan(&array).Error

	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("fields", fmt.Sprintf("%d", projectId)),
			zap.String("error", err.Error()))
		return nil,  errors.DataBaseError
	}

	vedioArray := []DeviceStatisticsItem{}
	sql = fmt.Sprintf("select t3.device_code as type_code, sum(1) as total, sum(t1.Alive=1) as online from VSSChannelTbl as t1 left join VSSDevTbl as t2 on t1.DevPubID=t2.DevPubID left join t_device as t3 on t3.sn=t1.DevPubID where t3.device_code=%d and t3.verify_status=1 and t3.project_id=? group by t3.device_code", DeviceTypeVedio)
	err = db.Raw(sql, projectId).Scan(&vedioArray).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", sql),
			zap.String("fields", fmt.Sprintf("%d", projectId)),
			zap.String("error", err.Error()))
		return nil,  errors.DataBaseError
	}
	if len(vedioArray) > 0 {
		return append(array, vedioArray...), nil
	}

	return array, nil
}

type IotType struct {
	Code int32
	Name string
	Id int64
}
func IotTypeList(db *gorm.DB, projectId int64, codes []int32) ([]IotType, error) {
	result := []IotType{}
	sql := fmt.Sprintf("select t1.device_code as code, t2.device_type as name, t2.id from t_device as t1 left join t_device_type_item as t2 on t1.device_code=t2.device_code where t1.project_id = %d and t1.verify_status = 1 and t1.device_code in(?) group by t1.device_code", projectId)
	err := db.Raw(sql, codes).Scan(&result).Error
	if err != nil {
		return nil, errors.DataBaseError
	}
	return result, nil
}
